@@ -341,6 +341,26 @@ static void CopyPatternName(CPattern &pattern, FileReader &file)
 }
 
 
+// Get version of Impulse Tracker that was used to create an IT/S3M file.
+mpt::ustring CSoundFile::GetImpulseTrackerVersion(uint16 cwtv, uint16 cmwt)
+{
+	mpt::ustring version;
+	cwtv &= 0xFFF;
+	if(cmwt > 0x0214)
+	{
+		version = UL_("Impulse Tracker 2.15");
+	} else if(cwtv >= 0x0215 && cwtv <= 0x0217)
+	{
+		const mpt::uchar *versions[] = {UL_("1-2"), UL_("3"), UL_("4-5")};
+		version = MPT_UFORMAT("Impulse Tracker 2.14p{}")(mpt::ustring_view(versions[cwtv - 0x0215]));
+	} else
+	{
+		version = MPT_UFORMAT("Impulse Tracker {}.{}")((cwtv & 0x0F00) >> 8, mpt::ufmt::hex0<2>((cwtv & 0xFF)));
+	}
+	return version;
+}
+
+
 // Get version of Schism Tracker that was used to create an IT/S3M file.
 mpt::ustring CSoundFile::GetSchismTrackerVersion(uint16 cwtv, uint32 reserved)
 {
@@ -778,6 +798,19 @@ bool CSoundFile::ReadIT(FileReader &file, ModLoadingFlags loadFlags)
 
 	bool possibleXMconversion = false;
 
+	// There's a bug in IT somewhere that resets the "sample data present" flag in sample headers, but keeps the sample length
+	// of a previously deleted sample (presumably).
+	// As old ModPlug versions didn't set this flag under some circumstances (if a sample wasn't referenced by any instruments in instrument mode),
+	// and because there appear to be some external tools that forget to set this flag at all, we only respect the flag if the file
+	// vaguely looks like it was saved with IT. Some files that play garbage data if we don't do this:
+	// astral projection.it by Lord Jon Ray
+	// classic illusions.it by Blackstar
+	// deep in dance.it by Simply DJ
+	// There are many more such files but they don't reference the broken samples in their pattern data, or the sample data pointer
+	// points right to the end of the file, so in both cases no audible problem can be observed.
+	const bool muteBuggySamples = !interpretModPlugMade && fileHeader.cwtv >= 0x0100 && fileHeader.cwtv <= 0x0217
+		&& (fileHeader.cwtv < 0x0207 || fileHeader.reserved != 0);
+
 	// Reading Samples
 	m_nSamples = std::min(static_cast<SAMPLEINDEX>(fileHeader.smpnum), static_cast<SAMPLEINDEX>(MAX_SAMPLES - 1));
 	bool lastSampleCompressed = false, anyADPCM = false;
@@ -786,9 +819,10 @@ bool CSoundFile::ReadIT(FileReader &file, ModLoadingFlags loadFlags)
 		ITSample sampleHeader;
 		if(smpPos[i] > 0 && file.Seek(smpPos[i]) && file.ReadStruct(sampleHeader))
 		{
-			// IT does not check for the IMPS magic, and some bad XM->IT converter out there doesn't write the magic bytes for empty sample slots.
 			ModSample &sample = Samples[i + 1];
 			size_t sampleOffset = sampleHeader.ConvertToMPT(sample);
+			if(muteBuggySamples && !(sampleHeader.flags & ITSample::sampleDataPresent))
+				sample.nLength = 0;
 
 			m_szNames[i + 1] = mpt::String::ReadBuf(mpt::String::spacePadded, sampleHeader.name);
 
@@ -1027,6 +1061,7 @@ bool CSoundFile::ReadIT(FileReader &file, ModLoadingFlags loadFlags)
 
 		auto patData = Patterns[pat].begin();
 		ROWINDEX row = 0;
+		ModCommand dummy{};
 		while(row < numRows && patternData.CanRead(1))
 		{
 			uint8 b = patternData.ReadUint8();
@@ -1057,7 +1092,6 @@ bool CSoundFile::ReadIT(FileReader &file, ModLoadingFlags loadFlags)
 			}
 
 			// Now we grab the data for this particular row/channel.
-			ModCommand dummy{};
 			ModCommand &m = ch < m_nChannels ? patData[ch] : dummy;
 
 			if(chnMask[ch] & 0x10)
@@ -1138,8 +1172,18 @@ bool CSoundFile::ReadIT(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				const auto [command, param] = patternData.ReadArray<uint8, 2>();
 				S3MConvert(m, command, param, true);
+
+				// IT 1.xx does not support high offset command
+				if(m.command == CMD_S3MCMDEX && (m.param & 0xF0) == 0xA0 && fileHeader.cwtv < 0x0200)
+					m.command = CMD_DUMMY;
+				// Fix handling of commands V81-VFF in ITs made with old Schism Tracker versions
+				// (fixed in https://github.com/schismtracker/schismtracker/commit/ab5517d4730d4c717f7ebffb401445679bd30888 - one of the last versions to identify as v0.50)
+				else if(m.command == CMD_GLOBALVOLUME && m.param > 0x80 && fileHeader.cwtv >= 0x1000 && fileHeader.cwtv <= 0x1050)
+					m.param = 0x80;
+
 				// In some IT-compatible trackers, it is possible to input a parameter without a command.
-				// In this case, we still need to update the last value memory. OpenMPT didn't do this until v1.25.01.07.
+				// In this case, we still need to update the last value memory (so that we don't reuse a previous non-empty effect).
+				// OpenMPT didn't do this until v1.25.01.07.
 				// Example: ckbounce.it
 				lastValue[ch].command = m.command;
 				lastValue[ch].param = m.param;
@@ -1227,21 +1271,13 @@ bool CSoundFile::ReadIT(FileReader &file, ModLoadingFlags loadFlags)
 			} else if(fileHeader.cwtv == 0 && madeWithTracker.empty())
 			{
 				madeWithTracker = U_("Unknown");
+			} else if(fileHeader.cwtv >= 0x0208 && fileHeader.cwtv <= 0x0214 && !fileHeader.reserved && m_FileHistory.empty() && madeWithTracker.empty())
+			{
+				// Any file made with IT starting from v2.07 onwards should have an edit history
+				madeWithTracker = UL_("Unknown");
 			} else if(fileHeader.cmwt < 0x0300 && madeWithTracker.empty())
 			{
-				if(fileHeader.cmwt > 0x0214)
-				{
-					madeWithTracker = U_("Impulse Tracker 2.15");
-				} else if(fileHeader.cwtv > 0x0214)
-				{
-					// Patched update of IT 2.14 (0x0215 - 0x0217 == p1 - p3)
-					// p4 (as found on modland) adds the ITVSOUND driver, but doesn't seem to change
-					// anything as far as file saving is concerned.
-					madeWithTracker = MPT_UFORMAT("Impulse Tracker 2.14p{}")(fileHeader.cwtv - 0x0214);
-				} else
-				{
-					madeWithTracker = MPT_UFORMAT("Impulse Tracker {}.{}")((fileHeader.cwtv & 0x0F00) >> 8, mpt::ufmt::hex0<2>((fileHeader.cwtv & 0xFF)));
-				}
+				madeWithTracker = GetImpulseTrackerVersion(fileHeader.cwtv, fileHeader.cmwt);
 				if(m_FileHistory.empty() && fileHeader.reserved != 0)
 				{
 					// Starting from  version 2.07, IT stores the total edit time of a module in the "reserved" field
@@ -1739,6 +1775,10 @@ bool CSoundFile::SaveIT(std::ostream &f, const mpt::PathString &filename, bool c
 
 		// Write pattern header
 		ROWINDEX writeRows = mpt::saturate_cast<uint16>(Patterns[pat].GetNumRows());
+		if(compatibilityExport)
+			writeRows = std::clamp(writeRows, ROWINDEX(32), ROWINDEX(200));
+		if(writeRows != Patterns[pat].GetNumRows())
+			AddToLog(LogWarning, MPT_UFORMAT("Warning: Pattern {} was resized from {} to {} rows.")(pat, Patterns[pat].GetNumRows(), writeRows));
 		uint16 writeSize = 0;
 		uint16le patinfo[4];
 		patinfo[0] = 0;
@@ -1755,11 +1795,12 @@ bool CSoundFile::SaveIT(std::ostream &f, const mpt::PathString &filename, bool c
 		// Maximum 7 bytes per cell, plus end of row marker, so this buffer is always large enough to cover one row.
 		std::vector<uint8> buf(7 * maxChannels + 1);
 
-		for(ROWINDEX row = 0; row < writeRows; row++)
+		const ROWINDEX readRows = std::min(writeRows, Patterns[pat].GetNumRows());
+		for(ROWINDEX row = 0; row < readRows; row++)
 		{
 			uint32 len = 0;
 			const ModCommand *m = Patterns[pat].GetpModCommand(row, 0);
-
+			bool writePatternBreak = (readRows < writeRows && row + 1 == readRows && !Patterns[pat].RowHasJump(row));
 			for(CHANNELINDEX ch = 0; ch < maxChannels; ch++, m++)
 			{
 				// Skip mptm-specific notes.
@@ -1813,6 +1854,12 @@ bool CSoundFile::SaveIT(std::ostream &f, const mpt::PathString &filename, bool c
 					S3MSaveConvert(*m, command, param, true, compatibilityExport);
 					if (command) b |= 8;
 				}
+				if(writePatternBreak && !(b & 8))
+				{
+					b |= 8;
+					command = 'C' ^ 0x40;
+					writePatternBreak = false;
+				}
 				// Packing information
 				if (b)
 				{
@@ -1895,11 +1942,25 @@ bool CSoundFile::SaveIT(std::ostream &f, const mpt::PathString &filename, bool c
 				break;
 			} else
 			{
-				dwPos += len;
-				writeSize += (uint16)len;
+				writeSize += static_cast<uint16>(len);
 				mpt::IO::WriteRaw(f, buf.data(), len);
 			}
+			if(writePatternBreak)
+			{
+				// Didn't manage to put a pattern break, so put it on the next row instead.
+				const uint8 patternBreak[] = {1 | IT_bitmask_patternChanEnabled_c, 8, 'C' ^ 0x40, 0};
+				mpt::IO::Write(f, patternBreak);
+				writeSize += sizeof(patternBreak);
+			}
 		}
+		if(readRows < writeRows)
+		{
+			// Invent empty rows at end (if we end up here, the pattern is very short and we don't have to care about writeSize overflowing the 16-bit limit)
+			writeSize += static_cast<uint16>(writeRows - readRows);
+			buf.assign(writeRows - readRows, 0);
+			mpt::IO::Write(f, buf);
+		}
+		dwPos += writeSize;
 
 		mpt::IO::SeekAbsolute(f, dwPatPos);
 		patinfo[0] = writeSize;
@@ -2607,14 +2668,6 @@ bool CSoundFile::LoadExtendedSongProperties(FileReader &file, bool ignoreChannel
 		m_nMixLevels = MixLevels::Original;
 	//m_dwCreatedWithVersion
 	//m_dwLastSavedWithVersion
-	//m_nSamplePreAmp
-	//m_nVSTiVolume
-	//m_nDefaultGlobalVolume
-	LimitMax(m_nDefaultGlobalVolume, MAX_GLOBAL_VOLUME);
-	//m_nRestartPos
-	//m_ModFlags
-	LimitMax(m_nDefaultRowsPerBeat, MAX_ROWS_PER_BEAT);
-	LimitMax(m_nDefaultRowsPerMeasure, MAX_ROWS_PER_BEAT);
 
 	return true;
 }
