@@ -49,7 +49,7 @@ struct STMSampleHeader
 			&& mptSmp.nLoopEnd != 0xFFFF)
 		{
 			mptSmp.uFlags = CHN_LOOP;
-			mptSmp.nLoopEnd = std::min(mptSmp.nLoopEnd, mptSmp.nLength);
+			mptSmp.nLength = std::max(mptSmp.nLoopEnd, mptSmp.nLength);
 		}
 	}
 };
@@ -230,14 +230,30 @@ bool CSoundFile::ReadSTM(FileReader &file, ModLoadingFlags loadFlags)
 		return true;
 
 	InitializeGlobals(MOD_TYPE_STM);
+	InitializeChannels();
 
 	m_songName = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, fileHeader.songname);
 
 	m_modFormat.formatName = U_("Scream Tracker 2");
 	m_modFormat.type = U_("stm");
-	m_modFormat.madeWithTracker = MPT_UFORMAT("Scream Tracker {}.{}")(fileHeader.verMajor, mpt::ufmt::dec0<2>(fileHeader.verMinor));
 	m_modFormat.charset = mpt::Charset::CP437;
 
+	if(!std::memcmp(fileHeader.trackerName, "!Scream!", 8))
+	{
+		if(fileHeader.verMinor >= 21)
+			m_modFormat.madeWithTracker = UL_("Scream Tracker 2.2 - 2.3 or compatible");
+		else
+			m_modFormat.madeWithTracker = MPT_UFORMAT("Scream Tracker {}.{} or compatible")(fileHeader.verMajor, mpt::ufmt::dec0<2>(fileHeader.verMinor));
+	}
+	else if(!std::memcmp(fileHeader.trackerName, "BMOD2STM", 8))
+		m_modFormat.madeWithTracker = UL_("BMOD2STM");
+	else if(!std::memcmp(fileHeader.trackerName, "WUZAMOD!", 8))
+		m_modFormat.madeWithTracker = UL_("Wuzamod");
+	else if(!std::memcmp(fileHeader.trackerName, "SWavePro", 8))
+		m_modFormat.madeWithTracker = UL_("SoundWave Pro");
+	else
+		m_modFormat.madeWithTracker = UL_("Unknown");
+
 	m_playBehaviour.set(kST3SampleSwap);
 
 	m_nSamples = 31;
@@ -258,13 +274,6 @@ bool CSoundFile::ReadSTM(FileReader &file, ModLoadingFlags loadFlags)
 	if(fileHeader.verMinor > 10)
 		m_nDefaultGlobalVolume = std::min(fileHeader.globalVolume, uint8(64)) * 4u;
 
-	// Setting up channels
-	for(CHANNELINDEX chn = 0; chn < 4; chn++)
-	{
-		ChnSettings[chn].Reset();
-		ChnSettings[chn].nPan = (chn & 1) ? 0x40 : 0xC0;
-	}
-
 	// Read samples
 	uint16 sampleOffsets[31];
 	for(SAMPLEINDEX smp = 1; smp <= 31; smp++)
@@ -461,6 +470,7 @@ bool CSoundFile::ReadSTX(FileReader &file, ModLoadingFlags loadFlags)
 		return true;
 
 	InitializeGlobals(MOD_TYPE_STM);
+	InitializeChannels();
 
 	m_songName = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, fileHeader.songName);
 
@@ -479,13 +489,6 @@ bool CSoundFile::ReadSTX(FileReader &file, ModLoadingFlags loadFlags)
 	m_nDefaultSpeed = initTempo >> 4;
 	m_nDefaultGlobalVolume = std::min(fileHeader.globalVolume, uint8(64)) * 4u;
 
-	// Setting up channels
-	for(CHANNELINDEX chn = 0; chn < 4; chn++)
-	{
-		ChnSettings[chn].Reset();
-		ChnSettings[chn].nPan = (chn & 1) ? 0x40 : 0xC0;
-	}
-
 	std::vector<uint16le> patternOffsets, sampleOffsets;
 	file.Seek(fileHeader.patTableOffset << 4);
 	file.ReadVector(patternOffsets, fileHeader.numPatterns);
