@@ -136,11 +136,11 @@ CSoundFile::CSoundFile() :
 #ifdef MODPLUG_TRACKER
 	m_bChannelMuteTogglePending.reset();
 
-	m_nDefaultRowsPerBeat = m_PlayState.m_nCurrentRowsPerBeat = (TrackerSettings::Instance().m_nRowHighlightBeats) ? TrackerSettings::Instance().m_nRowHighlightBeats : 4;
+	m_nDefaultRowsPerBeat = m_PlayState.m_nCurrentRowsPerBeat = (TrackerSettings::Instance().m_nRowHighlightBeats) ? TrackerSettings::Instance().m_nRowHighlightBeats : DEFAULT_ROWS_PER_BEAT;
 	m_nDefaultRowsPerMeasure = m_PlayState.m_nCurrentRowsPerMeasure = (TrackerSettings::Instance().m_nRowHighlightMeasures >= m_nDefaultRowsPerBeat) ? TrackerSettings::Instance().m_nRowHighlightMeasures : m_nDefaultRowsPerBeat * 4;
 #else
-	m_nDefaultRowsPerBeat = m_PlayState.m_nCurrentRowsPerBeat = 4;
-	m_nDefaultRowsPerMeasure = m_PlayState.m_nCurrentRowsPerMeasure = 16;
+	m_nDefaultRowsPerBeat = m_PlayState.m_nCurrentRowsPerBeat = DEFAULT_ROWS_PER_BEAT;
+	m_nDefaultRowsPerMeasure = m_PlayState.m_nCurrentRowsPerMeasure = DEFAULT_ROWS_PER_MEASURE;
 #endif // MODPLUG_TRACKER
 
 	MemsetZero(Instruments);
@@ -212,6 +212,7 @@ void CSoundFile::InitializeGlobals(MODTYPE type)
 	m_nResampling = SRCMODE_DEFAULT;
 	m_dwLastSavedWithVersion = Version(0);
 	m_dwCreatedWithVersion = Version(0);
+	m_nTempoMode = TempoMode::Classic;
 
 	SetMixLevels(MixLevels::Compatible);
 
@@ -594,6 +595,8 @@ bool CSoundFile::CreateInternal(FileReader file, ModLoadingFlags loadFlags)
 	for(SAMPLEINDEX nSmp = 1; nSmp <= m_nSamples; nSmp++)
 	{
 		ModSample &sample = Samples[nSmp];
+		LimitMax(sample.nLength, MAX_SAMPLE_LENGTH);
+		sample.SanitizeLoops();
 
 #ifdef MPT_EXTERNAL_SAMPLES
 		if(SampleHasPath(nSmp))
@@ -662,6 +665,8 @@ bool CSoundFile::CreateInternal(FileReader file, ModLoadingFlags loadFlags)
 	LimitMax(m_nDefaultRowsPerBeat, MAX_ROWS_PER_BEAT);
 	LimitMax(m_nDefaultRowsPerMeasure, MAX_ROWS_PER_BEAT);
 	LimitMax(m_nDefaultGlobalVolume, MAX_GLOBAL_VOLUME);
+	LimitMax(m_nSamplePreAmp, MAX_PREAMP);
+	LimitMax(m_nVSTiVolume, MAX_PREAMP);
 	if(!m_tempoSwing.empty())
 		m_tempoSwing.resize(m_nDefaultRowsPerBeat);
 
@@ -876,11 +881,12 @@ double CSoundFile::GetCurrentBPM() const
 		bpm = m_PlayState.m_nMusicTempo.ToDouble();
 	} else
 	{
-		//with other modes, we calculate it:
-		double ticksPerBeat = m_PlayState.m_nMusicSpeed * m_PlayState.m_nCurrentRowsPerBeat; //ticks/beat = ticks/row * rows/beat
-		double samplesPerBeat = m_PlayState.m_nSamplesPerTick * ticksPerBeat;                //samps/beat = samps/tick * ticks/beat
-		bpm =  m_MixerSettings.gdwMixingFreq / samplesPerBeat * 60;                          //beats/sec  = samps/sec  / samps/beat
-	}	                                                                                     //beats/min  =  beats/sec * 60
+		// With other modes, we calculate it:
+		ROWINDEX rowsPerBeat = m_PlayState.m_nCurrentRowsPerBeat ? m_PlayState.m_nCurrentRowsPerBeat : DEFAULT_ROWS_PER_BEAT;
+		double ticksPerBeat = m_PlayState.m_nMusicSpeed * rowsPerBeat;         // Ticks/beat = ticks/row * rows/beat
+		double samplesPerBeat = m_PlayState.m_nSamplesPerTick * ticksPerBeat;  // Samps/beat = samps/tick * ticks/beat
+		bpm =  m_MixerSettings.gdwMixingFreq / samplesPerBeat * 60;            // Beats/sec  = samps/sec  / samps/beat
+	}	                                                                       // Beats/min  =  beats/sec * 60
 
 	return bpm;
 }
@@ -1039,7 +1045,7 @@ void CSoundFile::ResetChannels()
 		chn.nLength = 0;
 		if(chn.dwFlags[CHN_ADLIB] && m_opl)
 		{
-			CHANNELINDEX c = static_cast<CHANNELINDEX>(std::distance(std::begin(m_PlayState.Chn), &chn));
+			CHANNELINDEX c = static_cast<CHANNELINDEX>(std::distance(m_PlayState.Chn.data(), &chn));
 			m_opl->NoteCut(c);
 		}
 	}
