@@ -509,7 +509,7 @@ public:
 	ResamplingMode m_nResampling; // Resampling mode (if overriding the globally set resampling)
 	int32 m_nRepeatCount = 0;     // -1 means repeat infinitely.
 	ORDERINDEX m_nMaxOrderPosition;
-	ModChannelSettings ChnSettings[MAX_BASECHANNELS];  // Initial channels settings
+	std::array<ModChannelSettings, MAX_BASECHANNELS> ChnSettings;  // Initial channels settings
 	CPatternContainer Patterns;
 	ModSequenceSet Order;  // Pattern sequences (order lists)
 protected:
@@ -589,8 +589,8 @@ public:
 		bool m_bPositionChanged = true; // Report to plugins that we jumped around in the module
 
 	public:
-		CHANNELINDEX ChnMix[MAX_CHANNELS]; // Index of channels in Chn to be actually mixed
-		ModChannel Chn[MAX_CHANNELS];      // Mixing channels... First m_nChannels channels are master channels (i.e. they are never NNA channels)!
+		std::array<CHANNELINDEX, MAX_CHANNELS> ChnMix;  // Index of channels in Chn to be actually mixed
+		std::array<ModChannel, MAX_CHANNELS> Chn;       // Mixing channels... First m_nChannels channels are master channels (i.e. they are never NNA channels)!
 
 		struct MIDIMacroEvaluationResults
 		{
@@ -699,13 +699,13 @@ public:
 
 	enum ModLoadingFlags
 	{
-		onlyVerifyHeader   = 0x00,
 		loadPatternData    = 0x01, // If unset, advise loaders to not process any pattern data (if possible)
 		loadSampleData     = 0x02, // If unset, advise loaders to not process any sample data (if possible)
-		loadPluginData     = 0x04, // If unset, plugin data is not loaded (and as a consequence, plugins are not instanciated).
-		loadPluginInstance = 0x08, // If unset, plugins are not instanciated.
+		loadPluginData     = 0x04, // If unset, plugin data is not loaded (and as a consequence, plugins are not instantiated).
+		loadPluginInstance = 0x08, // If unset, plugins are not instantiated.
 		skipContainer      = 0x10,
 		skipModules        = 0x20,
+		onlyVerifyHeader   = 0x40, // Do not combine with other flags!
 
 		// Shortcuts
 		loadCompleteModule = loadSampleData | loadPatternData | loadPluginData | loadPluginInstance,
@@ -978,6 +978,7 @@ public:
 	bool LoadExtendedSongProperties(FileReader &file, bool ignoreChannelCount, bool* pInterpretMptMade = nullptr);
 	void LoadMPTMProperties(FileReader &file, uint16 cwtv);
 
+	static mpt::ustring GetImpulseTrackerVersion(uint16 cwtv, uint16 cmwt);
 	static mpt::ustring GetSchismTrackerVersion(uint16 cwtv, uint32 reserved);
 
 	// Reads extended instrument properties(XM/IT/MPTM).
@@ -1285,7 +1286,7 @@ public:
 	// Resolve note/instrument combination to real sample index. Return value is guaranteed to be in [0, GetNumSamples()].
 	SAMPLEINDEX GetSampleIndex(ModCommand::NOTE note, uint32 instr) const noexcept;
 
-	uint32 MapMidiInstrument(uint8 program, uint16 bank, uint8 midiChannel, uint8 note, bool isXG, std::bitset<16> drumChns);
+	uint32 MapMidiInstrument(uint8 program, uint16 bank, uint8 midiChannel, uint8 note, bool isXG, std::bitset<32> drumChns);
 	size_t ITInstrToMPT(FileReader &file, ModInstrument &ins, uint16 trkvers);
 	std::pair<bool, bool> LoadMixPlugins(FileReader &file);
 #ifndef NO_PLUGINS
