@@ -200,6 +200,7 @@ CSoundFile::samplecount_t CSoundFile::ReadOneTick()
 {
 	const auto origMaxMixChannels = m_MixerSettings.m_nMaxMixChannels;
 	m_MixerSettings.m_nMaxMixChannels = 0;
+	ResetMixStat();
 	while(m_PlayState.m_nBufferCount)
 	{
 		auto framesToRender = std::min(m_PlayState.m_nBufferCount, samplecount_t(MIXBUFFERSIZE));
@@ -669,7 +670,7 @@ bool CSoundFile::ProcessRow()
 
 		// Reset channel values
 		ModCommand *m = Patterns[m_PlayState.m_nPattern].GetpModCommand(m_PlayState.m_nRow, 0);
-		for (ModChannel *pChn = m_PlayState.Chn, *pEnd = pChn + m_nChannels; pChn != pEnd; pChn++, m++)
+		for (ModChannel *pChn = m_PlayState.Chn.data(), *pEnd = pChn + m_nChannels; pChn != pEnd; pChn++, m++)
 		{
 			// First, handle some quirks that happen after the last tick of the previous row...
 			if(m_playBehaviour[KST3PortaAfterArpeggio]
@@ -1591,6 +1592,11 @@ void CSoundFile::ProcessArpeggio(CHANNELINDEX nChn, int32 &period, Tuning::NOTEI
 				uint8 note = (GetType() != MOD_TYPE_MOD) ? chn.nNote : static_cast<uint8>(GetNoteFromPeriod(period, chn.nFineTune, chn.nC5Speed));
 				if(GetType() & (MOD_TYPE_DBM | MOD_TYPE_DIGI))
 					tick += 2;
+
+				// SFX uses a 0-1-2-0-2-1 pattern (fixed at 6 ticks per row)
+				if(GetType() == MOD_TYPE_SFX && tick > 3)
+					tick ^= 3;
+
 				switch(tick % 3)
 				{
 				case 1: note += (chn.nArpeggio >> 4); break;
@@ -1613,7 +1619,7 @@ void CSoundFile::ProcessArpeggio(CHANNELINDEX nChn, int32 &period, Tuning::NOTEI
 					}
 					period = GetPeriodFromNote(note, chn.nFineTune, chn.nC5Speed);
 
-					if(GetType() & (MOD_TYPE_DBM | MOD_TYPE_DIGI | MOD_TYPE_PSM | MOD_TYPE_STM | MOD_TYPE_OKT))
+					if(GetType() & (MOD_TYPE_DBM | MOD_TYPE_DIGI | MOD_TYPE_PSM | MOD_TYPE_STM | MOD_TYPE_OKT | MOD_TYPE_SFX))
 					{
 						// The arpeggio note offset remains effective after the end of the current row in ScreamTracker 2.
 						// This fixes the flute lead in MORPH.STM by Skaven, pattern 27.
@@ -1637,7 +1643,7 @@ void CSoundFile::ProcessVibrato(CHANNELINDEX nChn, int32 &period, Tuning::RATIOT
 
 	if(chn.dwFlags[CHN_VIBRATO])
 	{
-		const bool advancePosition = !m_SongFlags[SONG_FIRSTTICK] || ((GetType() & (MOD_TYPE_IT | MOD_TYPE_MPT)) && !(m_SongFlags[SONG_ITOLDEFFECTS]));
+		const bool advancePosition = !m_SongFlags[SONG_FIRSTTICK] || ((GetType() & (MOD_TYPE_IT | MOD_TYPE_MPT | MOD_TYPE_MED)) && !(m_SongFlags[SONG_ITOLDEFFECTS]));
 
 		if(GetType() == MOD_TYPE_669)
 		{
@@ -1834,8 +1840,15 @@ void CSoundFile::ProcessSampleAutoVibrato(ModChannel &chn, int32 &period, Tuning
 					vdelta += Util::muldiv(period, fineUpTable[l & 0x03], 0x10000) - period;
 				}
 			}
-			period = (period + vdelta) / 256;
-			nPeriodFrac = vdelta & 0xFF;
+			if(Util::MaxValueOfType(period) - period >= vdelta)
+			{
+				period = (period + vdelta) / 256;
+				nPeriodFrac = vdelta & 0xFF;
+			} else
+			{
+				period = Util::MaxValueOfType(period) / 256;
+				nPeriodFrac = 0;
+			}
 		} else
 		{
 			// MPT's autovibrato code
@@ -2471,15 +2484,10 @@ bool CSoundFile::ReadNote()
 			{
 				int32 pan = (m_MixerSettings.gnChannels >= 2) ? Clamp(chn.nRealPan, 0, 256) : 128;
 
-				int32 realvol;
-				if(m_PlayConfig.getUseGlobalPreAmp())
-				{
-					realvol = (chn.nRealVolume * kChnMasterVol) / 128;
-				} else
-				{
-					// Extra attenuation required here if we're bypassing pre-amp.
-					realvol = (chn.nRealVolume * kChnMasterVol) / 256;
-				}
+				int32 realvol = (chn.nRealVolume * kChnMasterVol) / 128;
+				// Extra attenuation required here if we're bypassing pre-amp.
+				if(!m_PlayConfig.getUseGlobalPreAmp())
+					realvol /= 2;
 
 				const PanningMode panningMode = m_PlayConfig.getPanningMode();
 				if(panningMode == PanningMode::SoftPanning || (panningMode == PanningMode::Undetermined && (m_MixerSettings.MixerFlags & SNDMIX_SOFTPANNING)))
@@ -2500,6 +2508,7 @@ bool CSoundFile::ReadNote()
 					// you can never truly achieve 100% right panning in FT2, only 100% left.
 					// Test case: FT2PanLaw.xm
 					LimitMax(pan, 255);
+
 					const int panL = pan > 0 ? XMPanningTable[256 - pan] : 65536;
 					const int panR = XMPanningTable[pan];
 					chn.newLeftVol = (realvol * panL) / 65536;
@@ -2635,11 +2644,11 @@ void CSoundFile::ProcessMidiOut(CHANNELINDEX nChn)
 	// Check for volume commands
 	uint8 vol = 0xFF;
 	if(chn.rowCommand.volcmd == VOLCMD_VOLUME)
-		vol = std::min(chn.rowCommand.vol, uint8(64));
+		vol = std::min(chn.rowCommand.vol, uint8(64)) * 2u;
 	else if(chn.rowCommand.command == CMD_VOLUME)
-		vol = std::min(chn.rowCommand.param, uint8(64));
+		vol = std::min(chn.rowCommand.param, uint8(64)) * 2u;
 	else if(chn.rowCommand.command == CMD_VOLUME8)
-		vol = static_cast<uint8>((chn.rowCommand.param + 3u) / 4u);
+		vol = static_cast<uint8>((chn.rowCommand.param + 1u) / 2u);
 
 	const bool hasVolCommand = (vol != 0xFF);
 
@@ -2653,7 +2662,7 @@ void CSoundFile::ProcessMidiOut(CHANNELINDEX nChn)
 			SendMIDINote(nChn, realNote, static_cast<uint16>(chn.nVolume));
 		} else if(hasVolCommand)
 		{
-			pPlugin->MidiCC(MIDIEvents::MIDICC_Volume_Fine, vol, nChn);
+			pPlugin->MidiCC(MIDIEvents::MIDICC_Volume_Fine, vol / 2u, nChn);
 		}
 		return;
 	}
@@ -2667,7 +2676,7 @@ void CSoundFile::ProcessMidiOut(CHANNELINDEX nChn)
 		switch(pIns->pluginVelocityHandling)
 		{
 			case PLUGIN_VELOCITYHANDLING_CHANNEL:
-				velocity = chn.nVolume;
+				velocity = hasVolCommand ? vol * 2 : chn.nVolume;
 				break;
 			default:
 				break;
@@ -2695,11 +2704,11 @@ void CSoundFile::ProcessMidiOut(CHANNELINDEX nChn)
 		switch(pIns->pluginVolumeHandling)
 		{
 			case PLUGIN_VOLUMEHANDLING_DRYWET:
-				if(hasVolCommand) pPlugin->SetDryRatio(1.0f - (2 * vol) / 127.0f);
+				if(hasVolCommand) pPlugin->SetDryRatio(1.0f - vol / 127.0f);
 				else pPlugin->SetDryRatio(1.0f - (2 * defaultVolume) / 127.0f);
 				break;
 			case PLUGIN_VOLUMEHANDLING_MIDI:
-				if(hasVolCommand) pPlugin->MidiCC(MIDIEvents::MIDICC_Volume_Coarse, std::min(uint8(127), static_cast<uint8>(2 * vol)), nChn);
+				if(hasVolCommand) pPlugin->MidiCC(MIDIEvents::MIDICC_Volume_Coarse, std::min(uint8(127), vol), nChn);
 				else pPlugin->MidiCC(MIDIEvents::MIDICC_Volume_Coarse, static_cast<uint8>(std::min(uint32(127), static_cast<uint32>(2 * defaultVolume))), nChn);
 				break;
 			default:
