@@ -69,11 +69,7 @@ static_assert(CTuning::s_RatioTableFineSizeMaxDefault < static_cast<USTEPINDEXTY
 
 
 CTuning::CTuning()
-	: m_TuningType(Type::GENERAL)
-	, m_FineStepCount(0)
 {
-	m_RatioTable.clear();
-	m_NoteMin = s_NoteMinDefault;
 #if MPT_GCC_AT_LEAST(12, 0, 0) && MPT_GCC_BEFORE(13, 1, 0)
 // https://gcc.gnu.org/bugzilla/show_bug.cgi?id=109455
 #pragma GCC diagnostic push
@@ -83,9 +79,20 @@ CTuning::CTuning()
 #if MPT_GCC_AT_LEAST(12, 0, 0) && MPT_GCC_BEFORE(13, 1, 0)
 #pragma GCC diagnostic pop
 #endif
-	m_GroupSize = 0;
-	m_GroupRatio = 0;
-	m_RatioTableFine.clear();
+}
+
+
+bool CTuning::operator==(const CTuning &other) const noexcept
+{
+	return m_TuningType == other.m_TuningType
+		&& m_NoteMin == other.m_NoteMin
+		&& m_GroupSize == other.m_GroupSize
+		&& m_GroupRatio == other.m_GroupRatio
+		&& m_FineStepCount == other.m_FineStepCount
+		&& m_RatioTable == other.m_RatioTable
+		&& m_RatioTableFine == other.m_RatioTableFine
+		&& m_TuningName == other.m_TuningName
+		&& m_NoteNameMap == other.m_NoteNameMap;
 }
 
 
