@@ -50,7 +50,11 @@
 #elif defined(_MSC_VER)
 
 #define MPT_COMPILER_MSVC 1
-#if (_MSC_VER >= 1940)
+#if (_MSC_VER >= 1942)
+#define MPT_COMPILER_MSVC_VERSION MPT_COMPILER_MAKE_VERSION2(2022, 12)
+#elif (_MSC_VER >= 1941)
+#define MPT_COMPILER_MSVC_VERSION MPT_COMPILER_MAKE_VERSION2(2022, 11)
+#elif (_MSC_VER >= 1940)
 #define MPT_COMPILER_MSVC_VERSION MPT_COMPILER_MAKE_VERSION2(2022, 10)
 #elif (_MSC_VER >= 1939)
 #define MPT_COMPILER_MSVC_VERSION MPT_COMPILER_MAKE_VERSION2(2022, 9)
@@ -204,6 +208,16 @@
 
 
 
+// detect compiler quirks
+
+#if MPT_COMPILER_CLANG
+#if defined(__APPLE__)
+#define MPT_COMPILER_QUIRK_APPLE_CLANG
+#endif
+#endif
+
+
+
 // detect compiler setting quirks
 
 #if MPT_COMPILER_GCC
